/*
	RLTexture.h
	RedlineHackKit
	$Id$
	
	Class for extracting data from Redline texture (.txr) files. This does not handle binding the
	texture in OpenGL; this will be done by a higher layer or category when required.
	
	Copyright © 2007 Jens Ayton

	Permission is hereby granted, free of charge, to any person obtaining a copy of this software
	and associated documentation files (the “Software”), to deal in the Software without
	restriction, including without limitation the rights to use, copy, modify, merge, publish,
	distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
	Software is furnished to do so, subject to the following conditions:
	
	The above copyright notice and this permission notice shall be included in all copies or
	substantial portions of the Software.

	THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
	BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
	NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
	DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#import <Foundation/Foundation.h>
#import "RLHackKitCommon.h"

@class RLTextureMipLevel;


@interface RLTexture: NSObject
{
	NSData					*data;
	NSArray					*mipLevels;
}

+ (id)textureWithData:(NSData *)inData error:(NSError **)outError;
+ (id)textureWithURL:(NSURL *)inURL error:(NSError **)outError;
+ (id)textureWithFile:(NSString *)inFilePath error:(NSError **)outError;

- (id)initWithData:(NSData *)inData error:(NSError **)outError;
- (id)initWithURL:(NSURL *)inURL error:(NSError **)outError;
- (id)initWithFile:(NSString *)inFilePath error:(NSError **)outError;

// Note: the texture file format does not strictly guarantee an order to the mip levels. RLTexture does; the 0th member will be the lowest level (largest size), the n+1th level will be a higher level (smaller size) than the nth level. However, it is possible for levels to be skipped. I doubt Redline handles this elegantly.
- (uint32_t)mipLevelCount;
- (RLTextureMipLevel *)mipLevelAtIndex:(uint32_t)inLevel;
- (NSEnumerator *)mipLevelEnumerator;

- (NSSize)size;	// returns size of biggest mip level.

@end


@interface RLTextureMipLevel: NSObject
{
	uint32_t				unknown,
							target,
							level,
							internalFormat,
							width,
							height,
							border,
							format,
							type;
	
	NSData					*sharedData;
	void					*mipData;
}

// These correspond to the parameters to glTexImage2D.
- (uint32_t)target;
- (uint32_t)level;
- (uint32_t)internalFormat;
- (uint32_t)width;
- (uint32_t)height;
- (uint32_t)border;
- (uint32_t)format;
- (uint32_t)type;

- (NSSize)size;

// WARNING: this pointer will only be valid as long as the texture object and at least one mip level object are alive.
- (void *)bytes;
- (size_t)dataSize;	// Can potentially return 0 for unknown formats.

// WARNING: this data will only be valid as long as the texture object and at least one mip level object are alive.
- (NSData *)data;

@end
